#!/bin/bash

lib_dir=lib/

sjl_driver=codese.SJLDriver
codese_driver=codese.CoDeSeDriver

sjl_serialized=sjl-serialized
codese_serialized=codese-serialized

xclasspath=${codese_serialized}/codese-serialized.jar
for file in `find ${lib_dir} -name "*.jar"`; do
    xclasspath=${xclasspath}":"${file}
done

declare -A packages
packages=(
 ["arrayrvector_threads2"]="arrayrvector_threads2.org.apache.commons.math.linear"
 ["irbt_size10000_threads2"]="irbt_size10000_threads2.codese.dataset.intredblacktree"
 ["irbt_size1000_threads3"]="irbt_size1000_threads3.codese.dataset.intredblacktree"
 ["lmoptimizer_threads3"]="lmoptimizer_threads3.org.apache.commons.math.optimization.general"
 ["irbt_size10_threads1"]="irbt_size10_threads1.codese.dataset.intredblacktree"
 ["fasttreemap_threads1"]="fasttreemap_threads1.org.apache.commons.collections"
 ["fasttreemap_threads2"]="fasttreemap_threads2.org.apache.commons.collections"
 ["binheap_threads1"]="binheap_threads1.org.apache.commons.collections"
 ["bcelcode_threads1"]="bcelcode_threads1.org.aspectj.apache.bcel.classfile"
 ["bcelmethod_threads3"]="bcelmethod_threads3.org.aspectj.apache.bcel.classfile"
 ["dordmap_threads3"]="dordmap_threads3.org.apache.commons.collections"
 ["irbt_size10_threads2"]="irbt_size10_threads2.codese.dataset.intredblacktree"
 ["irbt_size100000_threads3"]="irbt_size100000_threads3.codese.dataset.intredblacktree"
 ["dordmap_threads1"]="dordmap_threads1.org.apache.commons.collections"
 ["irbt_size100_threads1"]="irbt_size100_threads1.codese.dataset.intredblacktree"
 ["instfactory_threads2"]="instfactory_threads2.org.aspectj.apache.bcel.generic"
 ["irbt_size1000_threads2"]="irbt_size1000_threads2.codese.dataset.intredblacktree"
 ["bcelmethod_threads1"]="bcelmethod_threads1.org.aspectj.apache.bcel.classfile"
 ["irbt_size10_threads3"]="irbt_size10_threads3.codese.dataset.intredblacktree"
 ["binheap_threads2"]="binheap_threads2.org.apache.commons.collections"
 ["irbt_size100000_threads2"]="irbt_size100000_threads2.codese.dataset.intredblacktree"
 ["irbt_size10000_threads1"]="irbt_size10000_threads1.codese.dataset.intredblacktree"
 ["arrayrvector_threads3"]="arrayrvector_threads3.org.apache.commons.math.linear"
 ["curvefitter_threads2"]="curvefitter_threads2.org.apache.commons.math.optimization.fitting"
 ["irbt_size100_threads3"]="irbt_size100_threads3.codese.dataset.intredblacktree"
 ["instfactory_threads3"]="instfactory_threads3.org.aspectj.apache.bcel.generic"
 ["lmoptimizer_threads2"]="lmoptimizer_threads2.org.apache.commons.math.optimization.general"
 ["instfactory_threads1"]="instfactory_threads1.org.aspectj.apache.bcel.generic"
 ["bcelcode_threads2"]="bcelcode_threads2.org.aspectj.apache.bcel.classfile"
 ["irbt_size100_threads2"]="irbt_size100_threads2.codese.dataset.intredblacktree"
 ["curvefitter_threads3"]="curvefitter_threads3.org.apache.commons.math.optimization.fitting"
 ["irbt_size10000_threads3"]="irbt_size10000_threads3.codese.dataset.intredblacktree"
 ["irbt_size1000_threads1"]="irbt_size1000_threads1.codese.dataset.intredblacktree"
 ["fasttreemap_threads3"]="fasttreemap_threads3.org.apache.commons.collections"
 ["dordmap_threads2"]="dordmap_threads2.org.apache.commons.collections"
 ["bcelmethod_threads2"]="bcelmethod_threads2.org.aspectj.apache.bcel.classfile"
 ["lmoptimizer_threads1"]="lmoptimizer_threads1.org.apache.commons.math.optimization.general"
 ["binheap_threads3"]="binheap_threads3.org.apache.commons.collections"
 ["curvefitter_threads1"]="curvefitter_threads1.org.apache.commons.math.optimization.fitting"
 ["bcelcode_threads3"]="bcelcode_threads3.org.aspectj.apache.bcel.classfile"
 ["irbt_size100000_threads1"]="irbt_size100000_threads1.codese.dataset.intredblacktree"
 ["arrayrvector_threads1"]="arrayrvector_threads1.org.apache.commons.math.linear"
)

function print_help() {
    find_subjects
    echo "./exe.sh [option arguments]"
    echo
    echo "        -a, --all"
    echo "                Deserialize all the subjects a given number of times (available subjects: ${subject_names})."
    echo
    echo "        -h, --help"
    echo "                Print this message."
    echo
    echo "        -o, --one"
    echo "                Deserialize a given subject a given number of times."
    echo
    echo "        -p, --paper"
    echo "                Deserialize all the subjects using the same numbers as in our study."
    echo
    echo "        -s, --send"
    echo "                Similar to -p; take info about cpu, memory, and java; create txt file."
    echo
    echo "EXAMPLES"
    echo "        ./exe.sh -a 1"
    echo "            Deserialize all ths subjects once."
    echo
    echo "        ./exe.sh -o irbt_size10000 2"
    echo "            Deserialize ArrayRVector two times."
    echo
}

threads=( 1 2 3 )
function execute_one() {
    subject_name=${1}
    num_of_iter=${2}

    sjl_file_name=${sjl_serialized}/${subject_name}.sjl
    sjl_result=`java -Xbootclasspath/p:${xclasspath} ${sjl_driver} ${subject_name} ${sjl_file_name} ${num_of_iter}`
    echo ${sjl_result}
    sjl_result_val=`echo ${sjl_result} | cut -d= -f2`
    for thread in ${threads[@]}; do
        codese_package_name=${packages[${subject_name}"_threads"${thread}]}
        codese_result=`java -Xbootclasspath/p:${xclasspath} ${codese_driver} ${subject_name}"_threads"${thread} ${codese_package_name} ${num_of_iter}`
        echo ${codese_result}
        codese_result_val=`echo ${codese_result} | cut -d= -f2`
        ratio=`echo "scale=2; ${sjl_result_val} / ${codese_result_val}" | bc`
        echo "Ratio_${subject_name}_threads${thread}_iter${num_of_iter}=${ratio}"
    done
    echo
}

function find_subjects() {
    subject_names=""
    for subject in `ls ${sjl_serialized}`; do
        subject_names=${subject_names}" "`basename ${subject} .sjl`
    done
}

function execute_all() {
    num_of_iter=$1

    for subject in `ls ${sjl_serialized}`; do
        execute_one `basename ${subject} .sjl` ${num_of_iter}
        echo ""
    done
}

function execute_paper() {
    execute_all 1
    execute_all 2
    execute_all 10
}

function execute_send_private() {
    echo "////////////////////////////// CPU //////////////////////////////"
    cat /proc/cpuinfo
    echo
    echo "////////////////////////////// MEMORY //////////////////////////////"
    cat /proc/meminfo
    echo
    echo "////////////////////////////// JAVA //////////////////////////////"
    java -version
    echo
    echo "////////////////////////////// RESULT //////////////////////////////"
    execute_paper
}

result_file="codese-study.txt"
function execute_send() {
    execute_send_private > ${result_file} 2>&1
}

case $1 in
    -a|--all) shift; execute_all "$@";;
    -o|--one) shift; execute_one "$@";;
    -p|--paper) shift; execute_paper "$@";;
    -s|--send) shift; execute_send "$@";;
    *) print_help;;
esac
