/**
 * University of Illinois/NCSA Open Source License
 *
 * Copyright (c) 2011 Milos Gligoric
 * All rights reserved.
 *
 * Developed by:  Milos Gligoric <milos.gligoric@gmail.edu>
 *                University of Illinois at Urbana-Champaign
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimers.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimers in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the names of Milos Gligoric, University of Illinois at Urbana-
 *    Champaign, nor the names of its contributors may be used to endorse
 *    or promote products derived from this Software without specific prior
 *    written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package codese;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

public class CoDeSeDriver {

  private static final String coDeSeDriverString = "CoDeSe";

  private static String subjectName;
  private static String packageName;
  private static int numOfIter;

  public static void main(String[] args) {
    try {
      subjectName = args[0];
      packageName = args[1];
      numOfIter = Integer.parseInt(args[2]);

      long time = System.currentTimeMillis();

      int countdown = numOfIter;
      CoDeSeInputStream dis = new CoDeSeInputStream(packageName);
      Object obj = null;
      obj = dis.readObject();
      while (countdown > 1) {
        obj = null;
        dis.reset();
        obj = dis.readObject();
        countdown--;
      }

      time = System.currentTimeMillis() - time;
      notifyFinished(obj, time);
    } catch (Exception ex) {
      ex.printStackTrace();
    }
  }

  private static void notifyFinished(Object obj, long time) {
    System.out.println(coDeSeDriverString + "_" + subjectName + "_iter"
        + numOfIter + "=" + time);
  }

  protected static void writeObjectToFile(Object obj, String fileName)
      throws IOException {
    if (obj == null)
      return;
    OutputStream os;
    os = new FileOutputStream(fileName);
    ObjectOutputStream oos = new ObjectOutputStream(os);
    oos.writeObject(obj);
    oos.flush();
    oos.close();
  }

}
