/**
 * University of Illinois/NCSA Open Source License
 *
 * Copyright (c) 2011 Milos Gligoric
 * All rights reserved.
 *
 * Developed by:  Milos Gligoric <milos.gligoric@gmail.edu>
 *                University of Illinois at Urbana-Champaign
 *
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal with the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 *
 *  * Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimers.
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimers in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the names of Milos Gligoric, University of Illinois at Urbana-
 *    Champaign, nor the names of its contributors may be used to endorse
 *    or promote products derived from this Software without specific prior
 *    written permission.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
 * OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 */

package codese;

import java.io.ObjectInputStream;
import java.lang.reflect.Method;

public class JavaUtilReadObjectMethods {

  private static Method javaUtilLinkedList;
  private static Method javaUtilArrayList;
  private static Method javaUtilHashMap;
  private static Method javaUtilTreeMap;

  public static Method javaUtilLinkedList() throws SecurityException,
      NoSuchMethodException {
    if (javaUtilLinkedList == null) {
      javaUtilLinkedList = readObjectMethod(java.util.LinkedList.class);
    }
    return javaUtilLinkedList;
  }

  public static Method javaUtilArrayList() throws SecurityException,
      NoSuchMethodException {
    if (javaUtilArrayList == null) {
      javaUtilArrayList = readObjectMethod(java.util.ArrayList.class);
    }
    return javaUtilArrayList;
  }

  public static Method javaUtilHashMap() throws SecurityException,
      NoSuchMethodException {
    if (javaUtilHashMap == null) {
      javaUtilHashMap = readObjectMethod(java.util.HashMap.class);
    }
    return javaUtilHashMap;
  }

  public static Method javaUtilTreeMap() throws SecurityException,
      NoSuchMethodException {
    if (javaUtilTreeMap == null) {
      javaUtilTreeMap = readObjectMethod(java.util.TreeMap.class);
    }
    return javaUtilTreeMap;
  }

  private static Method readObjectMethod(Class<?> clz)
      throws SecurityException, NoSuchMethodException {
    Method readObjectMethod = clz.getDeclaredMethod("readObject",
        ObjectInputStream.class);
    readObjectMethod.setAccessible(true);
    return readObjectMethod;
  }

}
